// For faster build
// Keep include order for VS compatibility.
#include "marisa/trie.cc"
// 1
#include "marisa/agent.cc"
// 2
#include "marisa/grimoire/io/reader.cc"
#include "marisa/grimoire/io/writer.cc"
// 3
#include "marisa/grimoire/io/mapper.cc"
#include "marisa/grimoire/trie/louds-trie.cc"
#include "marisa/grimoire/trie/tail.cc"
#include "marisa/grimoire/vector/bit-vector.cc"
#include "marisa/keyset.cc"
